

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
    *                                                                 *
    *   AutoAppend Proton Plugin 1.4                                  *
    *   Copyright  2004 Sebastian Pipping <webmaster@hartwork.org>   *
    *                                                                 *
    *   --> http://www.hartwork.org                                   *
    *                                                                 *
    *                                                                 *
    *   This source code is released under LGPL.                      *
    *   See LGPL.txt for details.                        2004-10-05   *
    *                                                                 *
    \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */



// CALL ORDER

// 01 dllmain
// 02   version
// 03   plugin
// 04   type
// 05 dllmain



#include <windows.h>

#include "devil_compile.h"
#include "devil_config.h"
#include "engine.h"
#include "escape.h"
#include "resource.h"


#define REPLACE_SELECTION	1
#define REPLACE_ALL			2



HINSTANCE g_hInstance = NULL;

DEVIL_CONFIG * g_CONFIG = NULL;
TCHAR * g_szAppend = NULL;

HWND g_hParent = NULL;
bool g_bInCommand = false;

HWND g_hDropbox  = NULL;
HWND g_hOK       = NULL;

UINT g_bEscape			= 0;
UINT g_bEscape_OLD		= 0;
UINT g_uHistoryCount	= 0;
UINT g_uHistoryLast		= 0;
UINT g_bShowDialog		= 0;
UINT g_bShowDialog_OLD	= 0;

bool g_bNullstring = true;

bool g_bShowDisableNotice = true;


int iReturn = 0;



/******************************************************************************************************
**  DllMain                                                                                          **
******************************************************************************************************/

BOOL APIENTRY DllMain( HINSTANCE hModule, DWORD dwReason, LPVOID lpReserved )
{
    if( dwReason == DLL_PROCESS_ATTACH )
	{
		g_hInstance = hModule;
	}

	return TRUE;
}


/******************************************************************************************************
**  wndproc_APPEND                                                                                   **
******************************************************************************************************/

BOOL CALLBACK wndproc_APPEND( HWND hwnd, UINT message, WPARAM wp, LPARAM lp )
{
	switch( message )
	{
		case WM_INITDIALOG:
		{
			// Handles
			g_hDropbox = GetDlgItem( hwnd, IDC_TEXT );
			g_hOK      = GetDlgItem( hwnd, IDOK ); 


			// Apply settings
			g_CONFIG->Read( "Escape",        ( int * )&g_bEscape,       0 );
			g_CONFIG->Read( "History_Count", ( int * )&g_uHistoryCount, 0 );
			g_CONFIG->Read( "History_Last",  ( int * )&g_uHistoryLast,  0 );
			g_CONFIG->Read( "Show_Dialog",   ( int * )&g_bShowDialog,   1 );

			g_bEscape_OLD      = g_bEscape;
			g_bShowDialog_OLD  = g_bShowDialog;

			CheckDlgButton( hwnd, IDC_ESCAPE,      ( g_bEscape     ? BST_CHECKED : BST_UNCHECKED ) );
			CheckDlgButton( hwnd, IDC_SHOW_DIALOG, ( g_bShowDialog ? BST_CHECKED : BST_UNCHECKED ) );

			if( g_uHistoryCount )
			{
				if( g_uHistoryLast > g_uHistoryCount )
				{
					g_uHistoryLast = g_uHistoryCount;
				}

				for( UINT u = g_uHistoryCount; u; u-- )
				{
					char szKey[ 20 ];
					wsprintf( szKey, "History_%u", u );
					char szString[ 256 ];
					g_CONFIG->Read( szKey, szString, "", 255 );

					if( !strlen( szString ) )
					{
						continue;
					}

					if( u == g_uHistoryLast )
					{
						SendMessage( g_hDropbox, WM_SETTEXT, 0, ( LPARAM )szString );
						g_bNullstring = false;
					}

					LRESULT res = SendMessage( ( HWND )g_hDropbox, ( UINT )CB_ADDSTRING, ( WPARAM )0, ( LPARAM )szString );
					if( ( res == CB_ERR ) || ( res == CB_ERRSPACE ) )
					{
						break;
					}
				}
			}
			else
			{
				g_bNullstring = true;
			}
			

			// Center window
			RECT r1;
			if( GetWindowRect( g_hParent, &r1 ) )
			{
				RECT r2;
				if( GetWindowRect( hwnd, &r2 ) )
				{
					const int w1 = r1.right - r1.left;
					const int h1 = r1.bottom - r1.top;
					const int w2 = r2.right - r2.left;
					const int h2 = r2.bottom - r2.top;

					const int x2 = r1.left + ( w1 - w2 ) / 2;
					const int y2 = r1.top + ( h1 - h2 ) / 2;

					MoveWindow( hwnd, x2, y2, w2, h2, TRUE );
				}
			}

			break;
		}
		case WM_COMMAND:
		{
			switch( LOWORD( wp ) )
			{
				case IDC_TEXT:
				{
					// Enable/disable "Append" button
					if( HIWORD( wp ) == CBN_EDITUPDATE )
					{
						UINT uLen = ( UINT )SendMessage( g_hDropbox, WM_GETTEXTLENGTH, 0, 0 );
						if( ( uLen == 0 ) ^ g_bNullstring )
						{
							g_bNullstring = !uLen;
							EnableWindow( g_hOK, ( BOOL )( uLen ? 1 : 0 ) );
						}
					}

					break;
				}
				case IDC_SHOW_DIALOG:
				{
					if( g_bShowDisableNotice && ( BST_UNCHECKED == IsDlgButtonChecked( hwnd, IDC_SHOW_DIALOG ) ) )
					{
						char szFinal[ 512 ];
						char szPath[ 256 ];
						g_CONFIG->get_inipath( szPath );
						wsprintf( szFinal, "You can re-enable this dialog by setting \"Show_Dialog\" to \"1\" in\n%s.", szPath );

						MessageBox( hwnd, szFinal, "AutoAppend", MB_ICONINFORMATION );

						g_bShowDisableNotice = false;
					}

					break;
				}
				case IDOK:
				{
					g_bEscape     = ( BST_CHECKED == IsDlgButtonChecked( hwnd, IDC_ESCAPE      ) );
					g_bShowDialog = ( BST_CHECKED == IsDlgButtonChecked( hwnd, IDC_SHOW_DIALOG ) );

					if( g_bEscape != g_bEscape_OLD )
					{
						g_CONFIG->Write( "Escape", ( int )g_bEscape );
						g_bEscape_OLD = g_bEscape;
					}
					if( g_bShowDialog != g_bShowDialog_OLD )
					{
						g_CONFIG->Write( "Show_Dialog", ( int )g_bShowDialog );
						g_bShowDialog_OLD = g_bShowDialog;
					}

					UINT uLen = ( UINT )SendMessage( g_hDropbox, WM_GETTEXTLENGTH, 0, 0 );
					if( uLen )
					{
						g_szAppend = new char[ uLen + 1 ];
						SendMessage( g_hDropbox, WM_GETTEXT, 255, ( LPARAM )g_szAppend );

						// New text?
						LRESULT res = SendMessage( g_hDropbox, CB_FINDSTRINGEXACT, ( WPARAM )( -1 ), ( LPARAM )g_szAppend );
						if( res == CB_ERR )
						{
							// Not found -> NEW
							g_uHistoryCount++;
							g_CONFIG->Write( "History_Count", ( int )g_uHistoryCount );
							g_CONFIG->Write( "History_Last", ( int )g_uHistoryCount );

							char szKey[ 20 ];
							wsprintf( szKey, "History_%u", g_uHistoryCount );
							g_CONFIG->Write( szKey, g_szAppend );
						}
						else
						{
							// Save index
							g_CONFIG->Write( "History_Last", ( int )( g_uHistoryCount - res ) );
						}
					}

					// NO BREAK !
				}
				case IDCANCEL:	// Button or [ESCAPE]
				{
					EndDialog( hwnd, LOWORD( wp ) );	// IDOK or IDCANCEL

					return TRUE;
				}
			}

			break;
		}
	}

	return 0;
}


/******************************************************************************************************
**  cleanup                                                                                          **
******************************************************************************************************/

char * cleanup()
{
	if( g_CONFIG != NULL )
	{
		delete g_CONFIG;
	}

	if( g_szAppend != NULL )
	{
		delete [] g_szAppend;
	}

	return NULL;
}


/******************************************************************************************************
**  execute                                                                                      @1  **
******************************************************************************************************/

char * __stdcall execute( char * szOriginal, int iSelStart, int iSelLen )
{
	// Config load
	g_CONFIG = new DEVIL_CONFIG( "AutoAppend", g_hInstance );
	g_CONFIG->Read( "Show_Dialog", ( int * )&g_bShowDialog, 1 );
	g_CONFIG->Read( "Escape",      ( int * )&g_bEscape,     0 );

	g_bEscape_OLD      = g_bEscape;
	g_bShowDialog_OLD  = g_bShowDialog;

	if( !g_bShowDialog )
	{
		g_CONFIG->Read( "History_Count", ( int * )&g_uHistoryCount, 0 );
		g_CONFIG->Read( "History_Last",  ( int * )&g_uHistoryLast,  0 );

		if( g_uHistoryCount )
		{
			if( g_uHistoryLast > g_uHistoryCount )
			{
				g_uHistoryLast = g_uHistoryCount;
			}

			char szKey[ 20 ];
			wsprintf( szKey, "History_%u", g_uHistoryLast );

			g_szAppend = new char[ 256 ];
			g_CONFIG->Read( szKey, g_szAppend, "", 255 );

			if( !strlen( g_szAppend ) )
			{
				delete [] g_szAppend;
				g_bShowDialog = 1;
			}
		}
		else
		{
			g_bShowDialog = 1;
		}
	}

	if( g_bShowDialog )
	{
		g_hParent = GetForegroundWindow();

		int res = DialogBox( g_hInstance, MAKEINTRESOURCE( IDD_APPEND ), g_hParent, wndproc_APPEND );

		if( ( res == IDCANCEL ) || ( g_szAppend == NULL ) || !strlen( g_szAppend ) )
		{
			return cleanup();
		}
	}


	if( g_bEscape != g_bEscape_OLD )
	{
		g_CONFIG->Write( "Escape", ( int )g_bEscape );
		g_bEscape_OLD = g_bEscape;
	}
	if( g_bShowDialog != g_bShowDialog_OLD )
	{
		g_CONFIG->Write( "Show_Dialog", ( int )g_bShowDialog );
		g_bShowDialog_OLD = g_bShowDialog;
	}


	// Decode escape sequences
	if( g_bEscape )
	{
		char * temp = NULL;
		decode_escape_sequences( g_szAppend, temp );
		delete [] g_szAppend;
		g_szAppend = temp;
	}
	
	
	// Process text
	char * out = NULL;
	if( iSelLen > 0 )
	{
		// Process selection
		char * in = new char[ iSelLen + 1 ];
		memcpy( in, szOriginal + iSelStart, iSelLen );
		in[ iSelLen ] = '\0';

		engine( in, out, g_szAppend, 2 );
		iReturn = REPLACE_SELECTION;

		delete [] in;
	}
	else
	{
		// Process whole file
		UINT in_len = strlen( szOriginal );
		if( in_len == 0 )
		{
			TCHAR in[] = "\0";
			engine( in, out, g_szAppend, 2 );
		}
		else
		{
			engine( szOriginal, out, g_szAppend, 2 );
		}

		iReturn = REPLACE_ALL;
	}


	// Finishing newline?
	UINT out_len = strlen( out );
	bool bNewlineFinish =	( out_len >= 2 ) &&
							( out[ out_len - 2 ] == '\x0D' ) &&
							( out[ out_len - 1 ] == '\x0A' );


	// Save last newline by doubling
	if( bNewlineFinish )
	{
		out[ out_len     ] = '\x0D';
		out[ out_len + 1 ] = '\x0A';
		out[ out_len + 2 ] = '\0';
	}


	cleanup();

	return out;
}


/******************************************************************************************************
**  get_return_type                                                                              @2  **
******************************************************************************************************/

int __stdcall get_return_type()
{
    return iReturn;
}

 
/******************************************************************************************************
**  get_interface_version                                                                        @3  **
******************************************************************************************************/

int __stdcall get_interface_version()
{
    return 1;
}
